# Commandos 1 - VOL Export
# by herbert3000 (ferdinand.graf.zeppelin@gmail.com), September 29, 2017
# copy this script into GIMP's plug-ins directory (usually C:\Program Files\GIMP 2\lib\gimp\2.0\plug-ins)
# Important: Delete all layers that should not be exported first

# Original script:
# Author: Chris Mohler <cr33dog@gmail.com>
# Copyright 2009 Chris Mohler
# "Only Visible" and filename formatting introduced by mh
# Thanks to Michael Holzt for layer group code!
# License: GPL v3+
# Version 0.6
# GIMP compatibilty 2.4.x -> 2.8.x
# GIMP plugin to export layers as PNGs

from gimpfu import *
import os, re

gettext.install("gimp20-python", gimp.locale_directory, unicode=True)

def format_filename(layer):
	layername = layer.name.decode('utf-8')
	regex = re.compile("[^-\w]", re.UNICODE) 
	filename = regex.sub('_', layername)
	return filename

def get_layers(layers):
	version = gimp.version[0:2]
	result = []
	only_visible = True
	for layer in layers:
		if version[0] >= 2 and version[1] >= 8: #version 2.8 and up
			if pdb.gimp_item_is_group(layer):
				result += get_layers(layer.children, only_visible)
			else:
				if only_visible:
					if layer.visible:
						layer.visible = False
						result.append(layer)
				else:
					layer.visible = False
					result.append(layer)
		else: #version below 2.8
			if only_visible:
				if layer.visible:
					layer.visible = False
					result.append(layer)
			else:
				layer.visible = False
				result.append(layer)
	
	return result

def export_layers(img, drw, path):
	write_cme_file = True
	write_txt_file = True
	write_images = True
	
	cme_file = None
	txt_file = None
	
	dupe = img.duplicate()
	layers = get_layers(dupe.layers)
	
	if write_txt_file:
		txt_path = os.path.join(path, "_Tiles.txt")
		txt_file = open(txt_path, "w")
	
	if write_cme_file:
		cme_path = os.path.join(path, "_Tiles.cme")
		cme_file = open(cme_path, "w")
		
		cme_file.writelines("CME1\n")
		cme_file.writelines("0\n") # bgx
		cme_file.writelines("0\n") # bgy
		cme_file.writelines("%d\n"% (img.width)) # mapx
		cme_file.writelines("%d\n"% (img.height)) # mapy
		cme_file.writelines("0\n") # back
		cme_file.writelines("%d\n"% (len(layers))) # nbrOfPolys
		cme_file.writelines("0\n") # selected
		cme_file.writelines("0\n") # polysel
		cme_file.writelines("0\n") # pos
	
	if write_images:
		fullpath = os.path.join(path, "png")
		if not os.path.exists(fullpath):
			os.makedirs(fullpath)
		fullpath = os.path.join(path, "bmp")
		if not os.path.exists(fullpath):
			os.makedirs(fullpath)
	
	for layer in reversed(layers):
		filename = format_filename(layer)
		
		x_off, y_off = layer.offsets
		
		if write_txt_file:
			txt_file.writelines("%d,%d,%d,%d,%s.png\n"% (x_off, y_off, layer.width, layer.height, filename))
		
		if write_cme_file:
			write_cme_tile(cme_file, x_off, y_off, layer.width, layer.height, filename, path)
	
	if write_images:
		for layer in layers:
			layer.visible = True
			filename = format_filename(layer)
			
			x_off, y_off = layer.offsets
			
			tmp = dupe.duplicate()
			tmp.merge_visible_layers(0)
			
			fullpath = os.path.join(path, "png")
			fullpath = os.path.join(fullpath, filename + ".png")
			pdb.file_png_save(tmp, tmp.layers[0], fullpath, filename + ".png", 0, 9, 1, 1, 1, 1, 1)
			
			fullpath = os.path.join(path, "bmp")
			fullpath = os.path.join(fullpath, filename + ".bmp")
			pdb.file_bmp_save(tmp, tmp.layers[0], fullpath, filename + ".bmp")
				
			dupe.remove_layer(layer)
	
	if write_cme_file:
		cme_file.writelines("EOF\n")
		cme_file.close()
	
	if write_txt_file:
		txt_file.close()

def write_cme_tile(file, x, y, width, height, name, path):
	file.writelines("%s\n"% (name)) # name
	file.writelines("0\n") # disabled
	file.writelines("%d\n"% (x + 204)) # center_x
	file.writelines("%d\n"% (y + 32 + 112)) # center_y
	file.writelines("0\n") # center_z
	file.writelines("%d\n"% (0)) # height
	file.writelines("0\n") # radio
	file.writelines("0\n") # nbrOfPoints
	file.writelines("1\n") # nbrOfTiles
	
	file.writelines("%s\\bmp\\%s.bmp\n"% (path, name)) # tile_name
	file.writelines("%s.RLE\n"% (name)) # tile_name2
	file.writelines("1\n") # isRle
	file.writelines("0\n") # tile_hide
	file.writelines("0\n") # tile_disabled
	file.writelines("0\n") # tile_offx
	file.writelines("0\n") # tile_offy
	file.writelines("%d\n"% (width)) # width
	file.writelines("%d\n"% (height)) # height
	file.writelines("0\n") # mirror
	file.writelines("0\n") # flip
	file.writelines("0\n") # light
	file.writelines("%d\n"% (0)) # x pos
	file.writelines("%d\n"% (0)) # y pos

register(
	proc_name=("python-fu-export-layers-vol"),
	blurb=("Export Layers as PNG for VOL"),
	help=("Export layers as individual PNG files."),
	author=("Chris Mohler <cr33dog@gmail.com>"),
	copyright=("Chris Mohler"),
	date=("2009"),
	label=("as _PNG"),
	imagetypes=("*"),
	params=[
		(PF_IMAGE, "img", "Image", None),
		(PF_DRAWABLE, "drw", "Drawable", None),
		(PF_DIRNAME, "path", "Save PNGs here", os.getcwd()),
		],
	results=[],
	function=(export_layers), 
	menu=("<Image>/File/Export Layers for VOL"), 
	domain=("gimp20-python", gimp.locale_directory)
	)

main()
